<?php

namespace Drupal\text_with_title\Plugin\Field\FieldType;

use Drupal\text\Plugin\Field\FieldType\TextItemBase;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\Field\FieldStorageDefinitionInterface as StorageDefinition;

/**
 * Plugin implementation of the 'Text with Title' field type.
 *
 * @FieldType(
 *   id = "text_with_title_field",
 *   label = @Translation("Text with Title"),
 *   description = @Translation("A wysiwyg area with associated title."),
 *   default_widget = "text_with_title_widget",
 *   default_formatter = "text_with_title_formatter"
 * )
 */
class TextWithTitle extends TextItemBase {

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(StorageDefinition $storage) {
    $properties = parent::propertyDefinitions($storage);

    // 'value' and 'format' come from TextItemBase.
    // Add custom 'title' property.
    $properties['title'] = DataDefinition::create('string')
      ->setLabel(t('Title'))
      ->setRequired(FALSE);

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(StorageDefinition $storage) {
    return [
      'columns' => [
        'title' => [
          'type' => 'varchar',
          'length' => 2048,
        ],
        'value' => [
          'type' => 'text',
          'size' => 'big',
          'not null' => FALSE,
        ],
        'format' => [
          'type' => 'varchar',
          'length' => 255,
          'not null' => FALSE,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $title = $this->get('title')->getValue();
    $text = $this->get('value')->getValue();
    return empty($title) && empty($text);
  }

}
